import bpy
from bpy.types import Operator

from ..utils.sectionbox import get_sectionbox


class SECTIONBOX_OT_Elevation(Operator):
    bl_idname = "sectionbox.elevation"
    bl_label = "Elevation"
    bl_description = "Create an elevation from this section plane"
    bl_options = {"UNDO", "INTERNAL"}

    action: bpy.props.StringProperty()  # create, remove, update
    enabled: bpy.props.BoolProperty()

    @classmethod
    def poll(cls, context):
        return True

    @classmethod
    def description(cls, context, properties):
        if properties.enabled:
            if properties.action == "create":
                return "Create an elevation from this section plane"
            elif properties.action == "remove":
                return "Remove the elevation of this section plane"
            elif properties.action == "update":
                return "Update the elevation of this section plane"
        else:
            return "Create a cross section first to allow elevation generation"

        return cls.bl_description

    def execute(self, context):
        sb = get_sectionbox(context)
        plane = sb.planes[sb.active_plane]
        if self.action == "create":
            try:
                bpy.ops.preferences.addon_enable(module="render_freestyle_svg")
            except:
                self.report({"ERROR"}, "Freestyle SVG Exporter addon not found!")
                return {"CANCELLED"}

            if not self.create(context, sb, plane):
                return {"CANCELLED"}

        elif self.action == "remove":
            plane.delete_elevation(context)

        elif self.action == "update":
            if not self.create(context, sb, plane):
                return {"CANCELLED"}

        action_txt = self.action + "d"
        self.report({"INFO"}, f"Elevation {action_txt}!")

        return {"FINISHED"}

    def create(self, context, sb, plane):
        plane.delete_elevation(context)
        if not sb.elevation(context, sb.planes[sb.active_plane]):
            self.report({"ERROR"}, "No elevation possible! Make sure that there are visible objects behind the plane.")
            return None

        return True
