import bpy
from bpy.types import Operator

from ..utils.sectionbox import get_sectionbox


class SECTIONBOX_OT_ListAction(Operator):
    bl_idname = "sectionbox.list_action"
    bl_label = ""
    bl_options = {"UNDO", "INTERNAL"}

    action: bpy.props.StringProperty()  # add, remove, up, down

    @classmethod
    def description(cls, context, properties):
        action = properties.action
        if action == "add":
            return "Add section box\nIf other objects selected, they will be added to the active section box"
        elif action == "remove":
            return "Remove section box\nIf other objects selected, they will be removed from the active section box"
        elif action == "up":
            return "Move section box up"
        elif action == "down":
            return "Move section box down"

    def execute(self, context):
        scene = context.scene
        sb_id = scene.sectionbox.list_index
        sb = get_sectionbox(context, check_active=False)

        if self.action == "add":
            if (
                sb
                and context.view_layer.objects.active == sb.object
                and len(context.view_layer.objects.selected) > 1
            ):
                bpy.ops.sectionbox.manage_objects("INVOKE_DEFAULT", action="add")
            else:
                bpy.ops.sectionbox.create("INVOKE_DEFAULT")

        elif self.action == "remove":
            if (
                sb
                and context.view_layer.objects.active == sb.object
                and len(context.view_layer.objects.selected) > 1
            ):
                bpy.ops.sectionbox.manage_objects("INVOKE_DEFAULT", action="remove")
            else:
                if sb:
                    sb.delete(context)
                else:
                    scene.sectionbox.list.remove(sb_id)

        elif self.action == "up" and sb_id > 0:
            scene.sectionbox.list.move(sb_id, sb_id - 1)
            scene.sectionbox.list_index -= 1

        elif self.action == "down" and sb_id < len(context.scene.sectionbox.list) - 1:
            scene.sectionbox.list.move(sb_id, sb_id + 1)
            scene.sectionbox.list_index += 1

        return {"FINISHED"}
