import bpy
from bpy.types import Context, Operator, OperatorProperties

from ..utils.preferences import get_prefs
from ..utils.sectionbox import get_sectionbox


class SECTIONBOX_OT_PropContextMenuAction(Operator):
    bl_idname = "sectionbox.prop_context_menu_action"
    bl_label = ""
    bl_options = {"UNDO", "INTERNAL"}

    action: bpy.props.StringProperty()  # default, reset

    @classmethod
    def poll(cls, context):
        return (
            hasattr(context, "button_prop")
            and context.button_pointer.bl_rna.name == "SECTIONBOX_PG_SectionBox"
        )

    @classmethod
    def description(cls, context: Context, properties: OperatorProperties) -> str:
        if not cls.poll(context):
            return "Defaulting not available for this property"

        if properties.action == "default":
            return f"Set current value as default"
        elif properties.action == "reset":
            return f"Reset the property to its default value"

    def execute(self, context):
        prefs = get_prefs()
        sb = get_sectionbox(context)
        prop = context.button_prop
        value = getattr(context.button_pointer, prop.identifier)
        if self.action == "default":
            if prop.identifier == "method":
                setattr(prefs, "def_method", value)
            elif prop.identifier == "material":
                setattr(prefs, "def_material", value)
                prefs.export_def_material()
                sb.material = prefs.def_material
            else:
                setattr(
                    prefs.def_settings,
                    prop.identifier,
                    value,
                )
            self.report(
                {"INFO"}, f'"{prop.name.capitalize()}" set to "{value}" as default!'
            )

        elif self.action == "reset":
            try:
                if prop.identifier == "method":
                    setattr(sb, "method", getattr(prefs, "def_method"))
                elif prop.identifier == "material":
                    prefs.import_def_material()
                    setattr(sb, "material", getattr(prefs, "def_material"))
                else:
                    value = getattr(prefs.def_settings, prop.identifier)
                    setattr(sb, prop.identifier, value)
            except:
                pass
            else:
                self.report(
                    {"INFO"},
                    f'"{prop.name.capitalize()}" reset to its default value of "{value}"!',
                )

        return {"FINISHED"}
