import bpy

from ..utils.sectionbox import get_sectionbox


class SECTIONBOX_OT_TransformModal(bpy.types.Operator):
    bl_idname = "sectionbox.transform_modal"
    bl_label = "Transform Modal"
    bl_options = {"INTERNAL"}

    prev_transform = {
        "location": None,
        "rotation_euler": None,
        "scale": None,
    }

    @classmethod
    def poll(cls, context):
        return True

    def invoke(self, context, event):
        context.window_manager.modal_handler_add(self)
        return {"RUNNING_MODAL"}

    def modal(self, context, event):
        if not context.window_manager.sectionbox.running_modals:
            return {"CANCELLED"}

        sb = get_sectionbox(context, check_active=True)
        if sb and sb.cutter:
            updated = False
            for key, prev_value in self.prev_transform.items():
                value = getattr(sb.object, key)
                if prev_value is None:
                    self.prev_transform[key] = value.copy()
                elif prev_value != value:
                    setattr(sb.cutter, key, value)
                    self.prev_transform[key] = value.copy()
                    updated = True

            if updated:
                sb.update_planes_intersecting(context)
                for plane in sb.planes:
                    plane.update_transform(context)

        else:
            self.prev_transform = self.prev_transform.fromkeys(
                self.prev_transform.keys(), None
            )

        return {"PASS_THROUGH"}
