import bpy
from bpy.types import Operator, Menu

from ..utils.sectionbox import get_sectionbox


class SECTIONBOX_OT_View(Operator):
    bl_idname = "sectionbox.view"
    bl_label = ""
    bl_options = {"UNDO", "INTERNAL"}

    view_id: bpy.props.IntProperty()
    action: bpy.props.StringProperty()  # set, override, add, remove
    name: bpy.props.StringProperty(
        name="Name",
    )

    @classmethod
    def description(cls, context, properties):
        action = properties.action
        if action == "set":
            return "Set as active view"
        elif action == "override":
            return "Override the active view with the current settings"
        elif action == "add":
            return "Add a new view from the current settings"
        elif action == "remove":
            return "Remove the active view"

    def draw(self, context):
        layout = self.layout
        layout.prop(self, "name")

    def invoke(self, context, event):
        if self.action == "add":
            return context.window_manager.invoke_props_dialog(self)
        else:
            return self.execute(context)

    def execute(self, context):
        sb = get_sectionbox(context)
        if not sb:
            return {"CANCELLED"}

        if self.action == "set":
            sb.set_view(context, self.view_id)

        elif self.action == "override":
            if sb.active_view == -1:
                return {"CANCELLED"}
            sb.views[sb.active_view].init(sb)

        elif self.action == "add":
            sb.add_view(self.name)

        elif self.action == "remove":
            if sb.active_view == -1:
                return {"CANCELLED"}
            sb.remove_view(sb.active_view)

        return {"FINISHED"}


class SECTIONBOX_MT_ViewMenu(Menu):
    bl_label = "Views"
    bl_description = "Saved views of the active section box"

    @classmethod
    def poll(cls, context):
        return get_sectionbox(context)

    def draw(self, context):
        layout = self.layout
        sb = get_sectionbox(context)
        for i, view in enumerate(sb.views):
            props = layout.operator("sectionbox.view", text=view.name)
            props.view_id = i
            props.action = "set"
