import bpy
from bpy.types import PropertyGroup

from ..utils.decorators import update_decorator


@update_decorator
def update_transform(self, context):
    sb_obj = context.scene.sectionbox.list.get(self.sb_name)
    if sb_obj is None:
        return

    sb = sb_obj.object.sectionbox
    sb.update_transform(context)
    sb.update_cutter(context)


@update_decorator
def update_edge_transform(self, context):
    update_transform(self, context)


class SECTIONBOX_PG_SectionPlaneSettings(PropertyGroup):
    update_: bpy.props.BoolProperty()

    transform: bpy.props.FloatProperty(
        name=f"Plane Transform",
        description="Transform of the section plane",
        default=0.0,
        soft_min=0,
        update=update_transform,
    )
    edge_transform: bpy.props.FloatVectorProperty(
        name="Edge Transform",
        description="Transform of the section plane edges",
        size=4,
        default=(0.0, 0.0, 0.0, 0.0),
        soft_min=0.0,
        update=update_transform,
    )
