import bpy


def close_collection_callback(context, area, prev_display_mode):
    try:
        with context.temp_override(area=area):
            bpy.ops.outliner.show_one_level(open=False)
    except Exception as e:
        print(e)

    area.spaces[0].filter_text = ""
    area.spaces[0].display_mode = prev_display_mode


def close_collection(context, collection, close=True):
    for area in context.screen.areas:
        if area.type == "OUTLINER":
            area: bpy.types.Area
            area.spaces[0].filter_text = collection.name
            prev_display_mode = area.spaces[0].display_mode
            area.spaces[0].display_mode = "VIEW_LAYER"
            func = lambda: close_collection_callback(context, area, prev_display_mode)
            bpy.app.timers.register(func, first_interval=0.01)
            break

    return collection


def create_collection(context, name, parent):
    col = bpy.data.collections.get(name)
    if col:
        return col

    col = bpy.data.collections.new(name)
    parent.children.link(col)
    return col


def get_collection(context, name):
    col = bpy.data.collections.get(name)
    if not col:
        col = create_collection(context, name, context.scene.collection)
    return col


def get_sb_collection(context, sb):
    sbs_name = f"Section Boxes - {context.scene.name} - {context.view_layer.name}"
    sbs_col = bpy.data.collections.get(sbs_name)
    if not sbs_col:
        sbs_col = bpy.data.collections.new(sbs_name)
        context.scene.collection.children.link(sbs_col)

    sb_col = sbs_col.children.get(sb.sectionbox.name)
    if not sb_col:
        sb_col = bpy.data.collections.new(sb.name)
        sbs_col.children.link(sb_col)

    return sb_col


def get_plane_collection(context, plane):
    name = f"{plane.object.name}"
    col = bpy.data.collections.get(name)
    if not col:
        col = bpy.data.collections.new(name)
        sb_col = get_sb_collection(context, plane.sb)
        sb_col.children.link(col)

    return col


def get_child_collection(context, collection, name):
    col = bpy.data.collections.get(name)
    if not col:
        col = create_collection(context, name, parent=collection)

    return col


def get_selected_collections(context):
    return [c for c in context.selected_ids if c.bl_rna.identifier == "Collection"]


def link_to_collection(collection, objects=[], collections=[]):
    for col in collections:
        if col is collection:
            continue

        collection.children.link(col)

    for obj in objects:
        collection.objects.link(obj)


def unlink_from_collection(collection, objects=[], collections=[]):
    for obj in objects:
        collection.objects.unlink(obj)

    for col in collections:
        if col is collection:
            continue

        collection.children.unlink(col)
