import re
import numpy as np
from mathutils import Vector

from .common_funcs import point_tri_intersection_2d, line_line_intersection_2d


def load_svg(filepath):
    paths = []

    with open(filepath, mode="r") as f:
        lines = f.readlines()

    pattern = re.compile(r"^ *d=*")
    for l in [l for l in lines if re.match(pattern, l)]:
        path = []
        parts = l.split(" ")
        for p in parts:
            vert = p.split(",")
            try:
                path.append(Vector((float(vert[0]), float(vert[1]))))
            except:
                continue

        paths.append(path)

    vertices = []
    indices = []
    for path in paths:
        l = len(vertices)
        ids = [tuple(np.add(i, (l, l, l))) for i in ear_cutting_triangulation(path)]
        vertices.extend(path)
        indices.extend(ids)

    return (vertices, indices)


def ear_cutting_triangulation(vertices: list):
    verts = vertices.copy()
    indices = []

    def get_next(vert):
        return vert + 1 if vert < len(verts) - 1 else 0

    def get_prev(vert):
        return vert - 1 if vert > 0 else len(verts) - 1

    def is_ear(vert):
        prev = get_prev(vert)
        next = get_next(vert)

        for v in [v for v in verts if verts.index(v) not in (vert, prev, next)]:
            if point_tri_intersection_2d(v, verts[vert], verts[prev], verts[next]):
                return False

        if (
            len(verts) < 5
            or line_line_intersection_2d(verts[prev], verts[next], verts[get_prev(prev)], verts[get_prev((next))])
            or line_line_intersection_2d(verts[prev], verts[next], verts[get_next(prev)], verts[get_next(next)])
        ):
            return True

        return False

    while len(verts) != 2:
        for vert in range(len(verts)):
            if is_ear(vert):
                indices.append(
                    (vertices.index(verts[vert]), vertices.index(verts[vert - 1]), vertices.index(verts[vert + 1]))
                )
                verts.pop(vert)
                break

    return indices
