import bpy
import bmesh


def is_geo(object: bpy.types.Object):
    return object.type in {"MESH", "CURVE", "SURFACE", "META", "FONT"}


def get_realized_instances(context: bpy.types.Context, object: bpy.types.Object):
    dg = context.evaluated_depsgraph_get()
    bm = bmesh.new()
    mwi = object.matrix_world.inverted()
    # join instances
    for inst_obj in dg.object_instances:
        if inst_obj.parent and inst_obj.parent.original == object:
            if not is_geo(inst_obj.instance_object):
                continue

            mesh = inst_obj.instance_object.data
            bm.from_mesh(mesh)
            bmesh.ops.transform(
                bm,
                matrix=mwi @ inst_obj.matrix_world,
                verts=bm.verts[-len(mesh.vertices) :],
            )

    # join parent object
    if object.type == "MESH":
        bm.from_mesh(object.data)

    eval_mesh = bpy.data.meshes.new(f"{object.name}.evaluated")
    bm.to_mesh(eval_mesh)
    bm.free()
    eval_obj = bpy.data.objects.new(f"{object.name}.evaluated", eval_mesh)
    eval_obj.matrix_world = object.matrix_world
    context.scene.collection.objects.link(eval_obj)
    return eval_obj


def get_local_object(object: bpy.types.Object):
    local = None
    if object.library and is_geo(object):
        local = object.copy()
        local.data = object.data.copy()
        local.make_local()

    return local


def get_mesh_type(context: bpy.types.Context, object: bpy.types.Object):
    mesh_object = None
    if is_geo(object):
        mesh = object.evaluated_get(context.evaluated_depsgraph_get()).to_mesh()
        bm = bmesh.new()
        bm.from_mesh(mesh)
        new_mesh = bpy.data.meshes.new(object.name)
        bm.to_mesh(new_mesh)
        bm.free()
        mesh_object = bpy.data.objects.new(object.name, new_mesh)
        mesh_object.matrix_world = object.matrix_world

    return mesh_object


def get_evaluated_object(context: bpy.types.Context, object: bpy.types.Object):
    eval_obj = object
    if object.library:
        eval_obj = get_local_object(eval_obj)
        if eval_obj is None:
            return None

    if eval_obj.instance_type != "NONE":
        eval_obj = get_realized_instances(context, eval_obj)
        if eval_obj is None:
            return None

    if eval_obj.type != "MESH":
        eval_obj = get_mesh_type(context, eval_obj)
        if eval_obj is None:
            return None

    return eval_obj if eval_obj is not object else None
