import bpy


def sub_layout(layout: bpy.types.UILayout, spacing: tuple = (0, 0, 0), align: bool = False):
    """Create a sub layout with separators

    :param layout: Layout to create the sub layout in
    :type layout: bpy.types.UILayout
    :param spacing: Spacing of the layout in the order of top, left, right, defaults to (0, 0, 0)
    :type spacing: tuple, optional
    :param align: Align layout, defaults to False
    :type align: bool, optional
    :return: Column of the sub layout
    :rtype: bpy.types.UILayout
    """
    col = layout.column()
    col.separator(factor=spacing[0])
    row = col.row(align=align)
    row.separator(factor=spacing[1])
    col = row.column()
    row.separator(factor=spacing[2])
    return col


levels_icon = [
    ("RIGHTARROW", "DOWNARROW_HLT"),
    ("DISCLOSURE_TRI_RIGHT", "DISCLOSURE_TRI_DOWN"),
]


def expand_layout(
    layout: bpy.types.UILayout, name: str, level: int, data, expand_prop: str, enable: bool = True
) -> bpy.types.UILayout:
    """Create an expand layout

    :param layout: Layout to create the expand layout in
    :type layout: bpy.types.UILayout
    :param name: Name of the layout
    :type name: str
    :param level: Level of the layout: 0, 1 (different icons)
    :type level: int
    :param data: source data of the property
    :type data: AnyType
    :param expand_prop: expand property
    :type expand_prop: str
    :param enable: enable layout, defaults to True
    :type enable: bool, optional
    """
    row = layout.row(align=True)
    row.enabled = enable
    icon = levels_icon[level][1] if getattr(data, expand_prop) else levels_icon[level][0]
    row.prop(data, expand_prop, text="", icon=icon, emboss=False)
    row.label(text=name)
    return row


def split_layout(
    layout: bpy.types.UILayout,
    split_factor: float = 0.5,
    spacing: float = 0,
    align_left: bool = False,
    align_right: bool = False,
):
    split = layout.split(factor=split_factor)
    if spacing == 0:
        lcol = split.column(align=align_left)
    else:
        row = split.row()
        row.separator(factor=spacing)
        lcol = row.column(align=align_left)
    rcol = split.column(align=align_right)
    return lcol, rcol
